@@ -170,7 +170,7 @@
     (`((typename ,name)) name)
     (`((pointer-to) (typename ,name)) (string-append name "*"))
     ;;
-    (otherwise (error "missed" mspec-tail))))
+    (otherwise (error "1: missed" mspec-tail))))
 
 (define (mtype->ffi mspec-tail)
   (match mspec-tail
@@ -184,7 +184,7 @@
     (`((typename ,name)) name)
     (`((pointer-to) (typename ,name)) (string-append name "*"))
     ;;
-    (otherwise (error "missed" mspec-tail))))
+    (otherwise (error "2: missed" mspec-tail))))
 
 (define ffi-typemap
   '(("void" . ffi:void)
@@ -208,6 +208,15 @@
     (`((void)) 'ffi:void)
     ;;(`((pointer-to) ,type) 'ffi:pointer)
     (`((pointer-to) ,type) ''*)
+    (`((enum-def . ,rest2) . ,rest1) "ffi:int")
+    (`((typename ,name) . ,rest)
+     (let* ((udecl `(decl (decl-spec-list (typename ,name))
+			  (init-declr (ident "_"))))
+	    (udecl (expand-typerefs udecl *uddict* #:keep *ffi-keepers*))
+	    (mspec (udecl->mspec udecl))
+	    )
+       (simple-format #t "mspec: ~S\n" mspec)
+       (error "mspec->ffi-sym missed")))
     (otherwise (error "mspec->ffi-sym missed" mspec))))
 
 (define (mspec->ffi-wrapper mspec)
@@ -521,7 +530,6 @@
 	 )
     (set! *uddict* uddict)
     (set! *port* dport) ;; HACK
-
     (sf ";;\n")
     (sf ";; auto-generated by ffi-help.scm\n")
     (sf ";;\n")
@@ -544,10 +552,11 @@
 	 (udecl->ffi-decl (cdr pair) type-list))
 
 	((member (car pair) '(
-			      ;;"cairo_get_reference_count"
-			      ;;"cairo_set_user_data"
+			      "cairo_get_reference_count"
+			      "cairo_set_user_data"
+			      "cairo_status_t"
 			      "cairo_matrix_t"
-			      ;;"cairo_set_dash"
+			      "cairo_set_dash"
 			      "cairo_bool_t"
 			      "cairo_region_t"
 			      "cairo_create"
